# Confluent Proactive Support: Full Metrics Collector

# Overview

This repository contains the full-fledged metrics collector for pro-active support that collects a wider range of
metrics from Confluent customers.


# Development

## Requirements

This project requires Kafka 0.9 built against Scala 2.11, which as of 02-Nov-2015 is not yet officially released.
You must therefore manually build Kafka and install it to your local Maven repository:

```shell
# Install Kafka trunk/master (w/ Scala 2.11) to local maven directory
$ git clone git@github.com:confluentinc/kafka.git && cd kafka
$ ./gradlew -PscalaVersion=2.11.7 clean install
```

Also, this project requires [support-metrics-common](https://github.com/confluentinc/support-metrics-common),
which you may need to build and install locally prior to running the build for this project.


## Building

This project uses the standard maven lifecycles such as:

```shell
$ mvn compile
$ mvn test
$ mvn package # creates the jar file
```

**Important note for IDE users (e.g. IntelliJ IDEA, Eclipse)**:

You must your IDE to run each test class in its own JVM.  This ensures that tests that verify Kafka's metrics will
work  properly when run in combination with other tests in this project's test suite.

> Note: If you run the tests via e.g. `mvn test` on the command line then you get this behavior by default.

If you are using IntelliJ IDEA you must modify the Run and/or Debug configurations for your test runs to ensure
that IDEA launches a new JVM for each test class:

* _Run > Edit Configurations... > (select the test run(s) you need) > Configuration tab > Fork mode: "class".

Background: The reason (we think) is that Kafka's metrics (as of 0.9.0.0) are stored at a static level because of how
Yammer Metrics works.  Example code from Yammer Metrics:

```java
private static final MetricsRegistry DEFAULT_REGISTRY = new MetricsRegistry()
```

What we found is that, if we do not separate tests that make use of `EmbeddedKafkaCluster` by class and then run each
test class in its own JVM, then metrics from a previous cluster-based test might stick around for the next
cluster-based test, which may cause the latter cluster-based test(s) to fail.  Note that, until we find a better fix,
this requires us to have separate test classes for each test that makes use of `EmbeddedKafkaCluster` because some of
our tests actually do need to verify Kafka's metrics (test suites that do not contain tests that verify Kafka's metrics
do not suffer from this restriction).


## Packaging and releasing

By convention we create release branches of the same name as Kafka _version_ they are integrating with (cf.
`KAFKA_VERSION` in [settings.sh](https://github.com/confluentinc/packaging/blob/master/settings.sh) of
[packaging](https://github.com/confluentinc/packaging/)).

> Note on packaging branches: Unlike other CP projects this project does not require or make use of `archive`, `debian`,
> or `rpm` branches for packaging.

For example, the code of this project for collecting metrics from Apache Kafka version `0.9.0.0` must be maintained
in a shared branch named `0.9.0.0`.  However, this project's maven `<version>` defined [pom.xml](pom.xml) must match
`CONFLUENT_VERSION` in [settings.sh](https://github.com/confluentinc/packaging/blob/master/settings.sh) (like other CP
projects such as [kafka-rest](https://github.com/confluentinc/kafka-rest/)).

```
Branch `0.9.0.0` => code to integrate with Apache Kafka version 0.9.0.0 release
           |
           |
           +-- /pom.xml (top-level)
                   |
                   | defines
                   |
                   V
                <project>
                  ...
                  <version>2.0.0</version>  => for Confluent Platform 2.0.0 release
                  ...
                </project>
```
